# -*- coding: utf-8 -*-

from os.path import join, dirname

from django.conf import settings
from django.conf.urls.defaults import *
from django.contrib import admin
from django.http import HttpResponse, HttpResponseServerError
from django.template.context import RequestContext, Context

# Handle critical errors

def error500(request, template_name='500.html'):
    try:
        output = render_to_string(template_name, {}, RequestContext(request))
    except:
        output = "Critical error. Administrator was notified."
    return HttpResponseServerError(output)
handler500 = 'harness.urls.error500'

# Connect admin

admin.autodiscover()
urlpatterns = patterns('',
    (r'^admin/', include(admin.site.urls)),
) 

# Define debug-mode URLs

# static urls will be disabled in production mode,
# forcing user to configure httpd
if settings.DEBUG:
    urlpatterns += patterns('',
        url(r'^media/(.*)$',
            'django.views.static.serve',
            dict(document_root=settings.MEDIA_ROOT)
        ),
        url(r'^static/(.*)$',
            'django.views.static.serve',
            dict(document_root=settings.STATIC_ROOT)
        ),
        url(r'^admin-media/(.*)$',
            'django.views.static.serve',
            dict(document_root=join(dirname(admin.__file__), 'media'))
        ),
    )



# -*- coding: utf-8 -*-

"""
A Django site harness configuration by Andy Mikhailenko <andy at neithere dot net>.
Based on the Byteflow conf: http://hg.piranha.org.ua/byteflow/
The most recent version can be found at http://bitbucket.org/neithere/django-harness/
"""

#
# DO NOT EDIT THIS FILE!
#
# Please edit conf/settings.py to override attributes defined here or add your own ones.
#

import os.path
import sys

# Set project root relative to manage.py
PROJECT_ROOT = sys.path[0]

# This function is only used within this file
at_project_root = lambda name: os.path.join(PROJECT_ROOT, name)

# Define paths where to look for your Django apps (see documentation for details).
# Note that the entries are inserted in reversed order.
# The project root should remain in the first position.
for app_lookup_path in ('compat', 'apps'):
    sys.path.insert(1, at_project_root(app_lookup_path))

DEBUG = True

# Local time zone for this installation. All choices can be found here:
# http://www.postgresql.org/docs/8.1/static/datetime-keywords.html#DATETIME-TIMEZONE-SET-TABLE
TIME_ZONE = 'Europe/UK'

# Language code for this installation. All choices can be found here:
# http://www.w3.org/TR/REC-html40/struct/dirlang.html#langcodes
# http://blogs.law.harvard.edu/tech/stories/storyReader$15
LANGUAGE_CODE = 'en'

SITE_ID = 1

# These constants appear in default templates and are useful for basic SEO
SITE_NAME = SITE_KEYWORDS = SITE_DESCRIPTION = ''

# If you set this to False, Django will make some optimizations so as not
# to load the internationalization machinery.
USE_I18N = True

# Absolute path to the directory that holds media.
# Example: "/home/media/media.lawrence.com/"
MEDIA_ROOT  = at_project_root('media')
STATIC_ROOT = at_project_root('static')

# URL that handles the media served from MEDIA_ROOT.
# Example: "http://media.lawrence.com"
MEDIA_URL  = '/media/'
STATIC_URL = '/static/'

# URL prefix for admin media -- CSS, JavaScript and images. Make sure to use a
# trailing slash.
# Examples: "http://foo.com/media/", "/media/".
ADMIN_MEDIA_PREFIX = '/admin-media/'

# Don't share it with anybody
if not hasattr(globals(), 'SECRET_KEY'):
    SECRET_FILE = at_project_root('local_secret.txt')
    try:
        SECRET_KEY = open(SECRET_FILE).read().strip()
    except IOError:
        try:
            from random import choice
            SECRET_KEY = ''.join([choice('abcdefghijklmnopqrstuvwxyz0123456789!@#$%^&*(-_=+)') for i in range(50)])
            secret = file(SECRET_FILE, 'w')
            secret.write(SECRET_KEY)
            secret.close()
        except IOError:
            raise Exception('Please create a %s file with random characters to generate your secret key!' % SECRET_FILE)

# List of callables that know how to import templates from various sources.
TEMPLATE_LOADERS = (
    'django.template.loaders.filesystem.load_template_source',
    'django.template.loaders.app_directories.load_template_source',
)

MIDDLEWARE_CLASSES = (
    'django.middleware.common.CommonMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.http.SetRemoteAddrFromForwardedFor',
    'harness.middleware.ThreadIPMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'harness.middleware.ThreadUserMiddleware',
    'django.middleware.locale.LocaleMiddleware',
    'django.middleware.doc.XViewMiddleware',
    'django.contrib.flatpages.middleware.FlatpageFallbackMiddleware',
    'django.contrib.csrf.middleware.CsrfMiddleware',
)

TEMPLATE_CONTEXT_PROCESSORS = [
    "django.core.context_processors.auth",
    "django.core.context_processors.debug",
    "django.core.context_processors.i18n",
    "django.core.context_processors.media",
    "django.core.context_processors.request",
]

# Try using custom template context processors.
#
#   Example context_processors module:
#   >>> processors = ('my_vars',)
#   >>> def my_vars(request):
#   >>>     return {'foo': 'bar'}
try:
    from context_processors import processors
    for fname in processors:
        TEMPLATE_CONTEXT_PROCESSORS.append('context_processors.%s' % fname)
    del processors
except ImportError: pass

AUTHENTICATION_BACKENDS = (
    'django.contrib.auth.backends.ModelBackend',
)

ROOT_URLCONF = 'urls'

TEMPLATE_DIRS = (
    at_project_root('templates'),
)

INSTALLED_APPS = (
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'django.contrib.admin',
    'django.contrib.sitemaps',
    'django.contrib.flatpages',
    'django.contrib.markup',
)

# Override some constants with site-specific ones
try:
    from settings import *
except ImportError:
    import sys
    sys.stderr.write('Unable to read settings.py\n')
    sys.exit(1)

# Override even site-specific constants with installation-related ones
# (e.g. locally use a test SQLite database instead of MySQL on production server)
try:
    from settings_local import *
except ImportError: pass

# Warn if project is marked as designed for a version of Django different from current.
# This can give a hint if something gets broken.
try:
    ver = REQUIRE_DJANGO_VERSION
except:
    pass
else:
    assert isinstance(ver, tuple), 'DJANGO_VERSION must be a tuple, e.g. (1,0).'
    assert 1 < len(ver) < 3, 'DJANGO_VERSION tuple must contain between 1 and 3 integers'
    for i in range(0, len(ver)):
        assert isinstance(ver[i], int), 'Integer expected at DJANGO_VERSION[%d] but got %s' % (i, type(ver[i]))
    from django import VERSION, get_version
    if not ver == VERSION[:len(ver)]:
        str_ver = '.'.join(str(i) for i in ver)
        msg = 'This project was designed for Django %s. You have Django %s installed.\n'\
              'Please ensure there are no problems and update REQUIRE_DJANGO_VERSION '\
              'variable in project settings.' % (str_ver, get_version())
        from warnings import warn
        warn(msg, UserWarning)
    # cleanup
    del(VERSION)
    del(get_version)

try:
    MANAGERS = ADMINS
except NameError:
    raise NameError, "ADMINS variable must be specified in this form: (('Your Name', 'your_email@domain.com'),)"

try:
    TEMPLATE_DEBUG
except NameError:
    TEMPLATE_DEBUG = DEBUG

try:
    INSTALLED_APPS += ADDITIONAL_APPS
except NameError: pass

try:
    MIDDLEWARE_CLASSES += ADDITIONAL_MIDDLEWARE
except NameError: pass

try:
    TEMPLATE_CONTEXT_PROCESSORS += ADDITIONAL_TEMPLATE_CONTEXT_PROCESSORS
except NameError: pass

# Fix relative paths to SQLite databases under mod_python
if 'sqlite' in DATABASE_ENGINE  and  '/' not in DATABASE_NAME:
    DATABASE_NAME = at_project_root(DATABASE_NAME)



# -*- coding: utf-8 -*-
from harness.middleware import get_current_user, get_current_ip

def record_user(field_name='user'):
    """
    A signal handler for any model. Sets currently logged in user
    to a given field of the instance.

    Requires 'harness.middleware.ThreadUserMiddleware' to be loaded.

    Usage:

    >>> from django.db import models
    >>> from django.db.models.signals import pre_save
    >>> from django.contrib.auth.models import User
    >>> from harness.signals import set_user
    >>>
    >>> class Entry(models.Model):
    ...     text   = models.TextField()
    ...     author = models.ForeignKey(User, editable=False)
    >>>
    >>> pre_save.connect(set_user('author'), sender=Entry)
    """
    def f(sender, instance, **kw):
        if not field_name in instance._meta.get_all_field_names():
            raise AttributeError, 'Cannot set current user: no such field "%s" '\
                                  'in %s' % (field_name, instance._meta.object_name)
        if not getattr(instance, '%s_id' % field_name):
            user = get_current_user()
            assert user, 'get_current_user() must return User instance. '\
                                       'Check if middleware is connected'
            setattr(instance, field_name, user)
    return f

def record_ip(field_name='ip'):
    """
    A signal handler for any model. Sets current client's IP address
    to a given field of the instance.

    Requires 'harness.middleware.ThreadIPMiddleware' to be loaded.

    Usage: same as in record_user()
    """
    def f(sender, instance, **kw):
        if not field_name in instance._meta.get_all_field_names():
            raise AttributeError, 'Cannot set current IP: no such field "%s" '\
                                  'in %s' % (field_name, instance._meta.object_name)
        if not getattr(instance, field_name):
            ip = get_current_ip()
            assert ip, 'get_current_ip() must return IP address. '\
                                     'Check if middleware is connected'
            setattr(instance, field_name, ip)
    return f



# -*- coding: utf-8 -*-
import threading
_thread_locals = threading.local()

def get_current_ip():
    "Returns current IP address."
    return getattr(_thread_locals, 'ip', None)

def get_current_user():
    "Returns current user (authenticated or anonymous)."
    return getattr(_thread_locals, 'user', None)

class ThreadIPMiddleware(object):
    def process_request(self, request):
        _thread_locals.ip = request.META['REMOTE_ADDR']
        return None

class ThreadUserMiddleware(object):
    def process_request(self, request):
        _thread_locals.user = getattr(request, 'user', None)
        return None
